/*
 * Copyright (c) 2017 N. Billard.
 */

package com.massa.mrules.extensions.dsl.editor;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;

import com.massa.dsl.grammar.GrammarRepository;
import com.massa.log.Log;
import com.massa.log.LogFactory;
import com.massa.util.MBeanUtils;
import com.massa.util.StringUtils;

/**
 * Starts the spring boot application.
 */
@SpringBootApplication
public class MRulesDslEditorApplication {
	private static final Log LOG = LogFactory.getLog(MRulesDslEditorApplication.class.getName());

	public static void main(final String[] args) {
		SpringApplication.run(MRulesDslEditorApplication.class, args);
	}

	@Value("${grammar.loader.class:#{null}}")
	private String grammarLoaderClass;

	@Value("${grammars.to.load:#{null}}")
	private String grammarsToLoad;

	@Bean
	protected ServletContextListener listener() {
		return new ServletContextListener() {
			@Override
			public void contextInitialized(final ServletContextEvent sce) {
				MRulesDslEditorApplication.LOG.info("ServletContext initialized");

				try {
					MRulesDslEditorApplication.LOG.info("Loading grammars");

					if (MRulesDslEditorApplication.this.grammarLoaderClass != null && !StringUtils.isEmpty(MRulesDslEditorApplication.this.grammarLoaderClass)) {
						MBeanUtils.newInstance(MRulesDslEditorApplication.this.grammarLoaderClass, GrammarLoader.class).load();
					}

					if (MRulesDslEditorApplication.this.grammarsToLoad != null && !StringUtils.isEmpty(MRulesDslEditorApplication.this.grammarsToLoad)) {
						final String[] grammars = MRulesDslEditorApplication.this.grammarsToLoad.split("[,;|]");
						for (final String grammar: grammars) {
							final int last = grammar.lastIndexOf('/');
							if (last < 0) {
								GrammarRepository.loadGrammar("", grammar);
							} else {
								GrammarRepository.loadGrammar(grammar.substring(0, last), grammar.substring(last + 1));
							}
						}
					}

					MRulesDslEditorApplication.LOG.info("Grammars loaded");
				} catch (final Exception e) {
					MRulesDslEditorApplication.LOG.error("Failed to load grammars", e);
				}

			}

			@Override
			public void contextDestroyed(final ServletContextEvent sce) {
				MRulesDslEditorApplication.LOG.info("ServletContext destroyed");
			}
		};
	}

}
